@@ -1,30 +1,21 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
 using System.Reflection;
 using System.Runtime.InteropServices;
 
-namespace Python.Runtime {
-
+namespace Python.Runtime
+{
     /// <summary>
     /// Managed class that provides the implementation for reflected delegate
-    /// types. Delegates are represented in Python by generated type objects. 
-    /// Each of those type objects is associated an instance of this class, 
+    /// types. Delegates are represented in Python by generated type objects.
+    /// Each of those type objects is associated an instance of this class,
     /// which provides its implementation.
     /// </summary>
-
-    internal class DelegateObject : ClassBase {
-
+    internal class DelegateObject : ClassBase
+    {
         MethodBinder binder;
 
-        internal DelegateObject(Type tp) : base(tp) {
+        internal DelegateObject(Type tp) : base(tp)
+        {
             binder = new MethodBinder(tp.GetMethod("Invoke"));
         }
 
@@ -34,9 +25,11 @@ namespace Python.Runtime {
         // the true managed delegate the Python object represents (or null).
         //====================================================================
 
-        private static Delegate GetTrueDelegate(IntPtr op) {
+        private static Delegate GetTrueDelegate(IntPtr op)
+        {
             CLRObject o = GetManagedObject(op) as CLRObject;
-            if (o != null) {
+            if (o != null)
+            {
                 Delegate d = o.inst as Delegate;
                 return d;
             }
@@ -44,7 +37,8 @@ namespace Python.Runtime {
         }
 
 
-        internal override bool CanSubclass() {
+        internal override bool CanSubclass()
+        {
             return false;
         }
 
@@ -57,17 +51,20 @@ namespace Python.Runtime {
         // to the Python callable passed in.
         //====================================================================
 
-        public static IntPtr tp_new(IntPtr tp, IntPtr args, IntPtr kw) {
+        public static IntPtr tp_new(IntPtr tp, IntPtr args, IntPtr kw)
+        {
             DelegateObject self = (DelegateObject)GetManagedObject(tp);
 
-            if (Runtime.PyTuple_Size(args) != 1) {
+            if (Runtime.PyTuple_Size(args) != 1)
+            {
                 string message = "class takes exactly one argument";
                 return Exceptions.RaiseTypeError(message);
             }
 
             IntPtr method = Runtime.PyTuple_GetItem(args, 0);
 
-            if (Runtime.PyCallable_Check(method) != 1) {
+            if (Runtime.PyCallable_Check(method) != 1)
+            {
                 return Exceptions.RaiseTypeError("argument must be callable");
             }
 
@@ -76,24 +73,26 @@ namespace Python.Runtime {
         }
 
 
-
         //====================================================================
         // Implements __call__ for reflected delegate types.
         //====================================================================
 
-        public static IntPtr tp_call(IntPtr ob, IntPtr args, IntPtr kw) {
+        public static IntPtr tp_call(IntPtr ob, IntPtr args, IntPtr kw)
+        {
             // todo: add fast type check!
             IntPtr pytype = Runtime.PyObject_TYPE(ob);
             DelegateObject self = (DelegateObject)GetManagedObject(pytype);
             CLRObject o = GetManagedObject(ob) as CLRObject;
 
-            if (o == null) {
+            if (o == null)
+            {
                 return Exceptions.RaiseTypeError("invalid argument");
             }
-            
+
             Delegate d = o.inst as Delegate;
 
-            if (d == null) {
+            if (d == null)
+            {
                 return Exceptions.RaiseTypeError("invalid argument");
             }
             return self.binder.Invoke(ob, args, kw);
@@ -103,18 +102,46 @@ namespace Python.Runtime {
         //====================================================================
         // Implements __cmp__ for reflected delegate types.
         //====================================================================
+#if PYTHON3
+        public static new IntPtr tp_richcompare(IntPtr ob, IntPtr other, int op) {
+            if (op != Runtime.Py_EQ && op != Runtime.Py_NE)
+            {
+                Runtime.XIncref(Runtime.PyNotImplemented);
+                return Runtime.PyNotImplemented;
+            }
+
+            IntPtr pytrue = Runtime.PyTrue;
+            IntPtr pyfalse = Runtime.PyFalse;
+
+            // swap true and false for NE
+            if (op != Runtime.Py_EQ)
+            {
+                pytrue = Runtime.PyFalse;
+                pyfalse = Runtime.PyTrue;
+            }
 
-        public static new int tp_compare(IntPtr ob, IntPtr other) {
             Delegate d1 = GetTrueDelegate(ob);
             Delegate d2 = GetTrueDelegate(other);
-            if (d1 == d2) {
+            if (d1 == d2)
+            {
+                Runtime.XIncref(pytrue);
+                return pytrue;
+            }
+
+            Runtime.XIncref(pyfalse);
+            return pyfalse;
+        }
+#elif PYTHON2
+        public static new int tp_compare(IntPtr ob, IntPtr other)
+        {
+            Delegate d1 = GetTrueDelegate(ob);
+            Delegate d2 = GetTrueDelegate(other);
+            if (d1 == d2)
+            {
                 return 0;
             }
             return -1;
         }
-
-
-    }        
-
-
+#endif
+    }
 }
