@@ -1,27 +1,18 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
 using System.Runtime.InteropServices;
 using System.Collections;
 using System.Reflection;
 
-namespace Python.Runtime {
-
+namespace Python.Runtime
+{
     //========================================================================
     // The managed metatype. This object implements the type of all reflected
     // types. It also provides support for single-inheritance from reflected
     // managed types.
     //========================================================================
-    
-    internal class MetaType : ManagedType {
 
+    internal class MetaType : ManagedType
+    {
         static IntPtr PyCLRMetaType;
 
 
@@ -29,24 +20,27 @@ namespace Python.Runtime {
         // Metatype initialization. This bootstraps the CLR metatype to life.
         //====================================================================
 
-        public static IntPtr Initialize() {
+        public static IntPtr Initialize()
+        {
             PyCLRMetaType = TypeManager.CreateMetaType(typeof(MetaType));
             return PyCLRMetaType;
         }
 
 
         //====================================================================
-        // Metatype __new__ implementation. This is called to create a new 
-        // class / type when a reflected class is subclassed. 
+        // Metatype __new__ implementation. This is called to create a new
+        // class / type when a reflected class is subclassed.
         //====================================================================
 
-        public static IntPtr tp_new(IntPtr tp, IntPtr args, IntPtr kw) {
+        public static IntPtr tp_new(IntPtr tp, IntPtr args, IntPtr kw)
+        {
             int len = Runtime.PyTuple_Size(args);
-            if (len < 3) {
+            if (len < 3)
+            {
                 return Exceptions.RaiseTypeError("invalid argument list");
             }
 
-            //IntPtr name = Runtime.PyTuple_GetItem(args, 0);
+            IntPtr name = Runtime.PyTuple_GetItem(args, 0);
             IntPtr bases = Runtime.PyTuple_GetItem(args, 1);
             IntPtr dict = Runtime.PyTuple_GetItem(args, 2);
 
@@ -55,17 +49,18 @@ namespace Python.Runtime {
             // That type must itself have a managed implementation. We check
             // that by making sure its metatype is the CLR metatype.
 
-            if (Runtime.PyTuple_Size(bases) != 1) {
+            if (Runtime.PyTuple_Size(bases) != 1)
+            {
                 return Exceptions.RaiseTypeError(
-                       "cannot use multiple inheritance with managed classes"
-                       );
-
+                    "cannot use multiple inheritance with managed classes"
+                    );
             }
 
             IntPtr base_type = Runtime.PyTuple_GetItem(bases, 0);
             IntPtr mt = Runtime.PyObject_TYPE(base_type);
 
-            if (!((mt == PyCLRMetaType) || (mt == Runtime.PyTypeType))) {
+            if (!((mt == PyCLRMetaType) || (mt == Runtime.PyTypeType)))
+            {
                 return Exceptions.RaiseTypeError("invalid metatype");
             }
 
@@ -73,31 +68,44 @@ namespace Python.Runtime {
             // disallowing subclassing of delegates, enums and array types.
 
             ClassBase cb = GetManagedObject(base_type) as ClassBase;
-            if (cb != null) {
-                if (! cb.CanSubclass() ) {
+            if (cb != null)
+            {
+                if (!cb.CanSubclass())
+                {
                     return Exceptions.RaiseTypeError(
-                      "delegates, enums and array types cannot be subclassed"
-                      );
+                        "delegates, enums and array types cannot be subclassed"
+                        );
                 }
             }
 
             IntPtr slots = Runtime.PyDict_GetItemString(dict, "__slots__");
-            if (slots != IntPtr.Zero) {
+            if (slots != IntPtr.Zero)
+            {
                 return Exceptions.RaiseTypeError(
-                "subclasses of managed classes do not support __slots__"
-                );
+                    "subclasses of managed classes do not support __slots__"
+                    );
             }
 
-            // hack for now... fix for 1.0
-            //return TypeManager.CreateSubType(args);
-
-
-            // right way
+            // If __assembly__ or __namespace__ are in the class dictionary then create
+            // a managed sub type.
+            // This creates a new managed type that can be used from .net to call back
+            // into python.
+            if (IntPtr.Zero != dict)
+            {
+                Runtime.XIncref(dict);
+                using (PyDict clsDict = new PyDict(dict))
+                {
+                    if (clsDict.HasKey("__assembly__") || clsDict.HasKey("__namespace__"))
+                        return TypeManager.CreateSubType(name, base_type, dict);
+                }
+            }
 
+            // otherwise just create a basic type without reflecting back into the managed side.
             IntPtr func = Marshal.ReadIntPtr(Runtime.PyTypeType,
-                                             TypeOffset.tp_new);
+                TypeOffset.tp_new);
             IntPtr type = NativeCall.Call_3(func, tp, args, kw);
-            if (type == IntPtr.Zero) {
+            if (type == IntPtr.Zero)
+            {
                 return IntPtr.Zero;
             }
 
@@ -113,7 +121,7 @@ namespace Python.Runtime {
 
             // Hmm - the standard subtype_traverse, clear look at ob_size to
             // do things, so to allow gc to work correctly we need to move
-            // our hidden handle out of ob_size. Then, in theory we can 
+            // our hidden handle out of ob_size. Then, in theory we can
             // comment this out and still not crash.
             TypeManager.CopySlot(base_type, type, TypeOffset.tp_traverse);
             TypeManager.CopySlot(base_type, type, TypeOffset.tp_clear);
@@ -123,20 +131,19 @@ namespace Python.Runtime {
             IntPtr gc = Marshal.ReadIntPtr(base_type, TypeOffset.magic());
             Marshal.WriteIntPtr(type, TypeOffset.magic(), gc);
 
-            //DebugUtil.DumpType(base_type);
-            //DebugUtil.DumpType(type);
-
             return type;
         }
 
 
-        public static IntPtr tp_alloc(IntPtr mt, int n) {
+        public static IntPtr tp_alloc(IntPtr mt, int n)
+        {
             IntPtr type = Runtime.PyType_GenericAlloc(mt, n);
             return type;
         }
 
 
-        public static void tp_free(IntPtr tp) {
+        public static void tp_free(IntPtr tp)
+        {
             Runtime.PyObject_GC_Del(tp);
         }
 
@@ -147,39 +154,45 @@ namespace Python.Runtime {
         // from PyType_Type won't call __init__ for metatypes it doesnt know.
         //====================================================================
 
-        public static IntPtr tp_call(IntPtr tp, IntPtr args, IntPtr kw) {
+        public static IntPtr tp_call(IntPtr tp, IntPtr args, IntPtr kw)
+        {
             IntPtr func = Marshal.ReadIntPtr(tp, TypeOffset.tp_new);
-            if (func == IntPtr.Zero) {
+            if (func == IntPtr.Zero)
+            {
                 return Exceptions.RaiseTypeError("invalid object");
             }
-            
+
             IntPtr obj = NativeCall.Call_3(func, tp, args, kw);
-            if (obj == IntPtr.Zero) {
+            if (obj == IntPtr.Zero)
+            {
                 return IntPtr.Zero;
             }
 
             IntPtr py__init__ = Runtime.PyString_FromString("__init__");
             IntPtr type = Runtime.PyObject_TYPE(obj);
             IntPtr init = Runtime._PyType_Lookup(type, py__init__);
-            Runtime.Decref(py__init__);
+            Runtime.XDecref(py__init__);
             Runtime.PyErr_Clear();
 
-            if (init != IntPtr.Zero) {
+            if (init != IntPtr.Zero)
+            {
                 IntPtr bound = Runtime.GetBoundArgTuple(obj, args);
-                if (bound == IntPtr.Zero) {
-                    Runtime.Decref(obj);
+                if (bound == IntPtr.Zero)
+                {
+                    Runtime.XDecref(obj);
                     return IntPtr.Zero;
                 }
 
                 IntPtr result = Runtime.PyObject_Call(init, bound, kw);
-                Runtime.Decref(bound);
+                Runtime.XDecref(bound);
 
-                if (result == IntPtr.Zero) {
-                    Runtime.Decref(obj);
+                if (result == IntPtr.Zero)
+                {
+                    Runtime.XDecref(obj);
                     return IntPtr.Zero;
                 }
 
-                Runtime.Decref(result);
+                Runtime.XDecref(result);
             }
 
             return obj;
@@ -190,29 +203,41 @@ namespace Python.Runtime {
         // Type __setattr__ implementation for reflected types. Note that this
         // is slightly different than the standard setattr implementation for
         // the normal Python metatype (PyTypeType). We need to look first in
-        // the type object of a reflected type for a descriptor in order to 
+        // the type object of a reflected type for a descriptor in order to
         // support the right setattr behavior for static fields and properties.
         //====================================================================
 
-        public static int tp_setattro(IntPtr tp, IntPtr name, IntPtr value) {
+        public static int tp_setattro(IntPtr tp, IntPtr name, IntPtr value)
+        {
             IntPtr descr = Runtime._PyType_Lookup(tp, name);
 
-            if (descr != IntPtr.Zero) {
+            if (descr != IntPtr.Zero)
+            {
                 IntPtr dt = Runtime.PyObject_TYPE(descr);
-                IntPtr fp = Marshal.ReadIntPtr(dt, TypeOffset.tp_descr_set);
-                if (fp != IntPtr.Zero) {
-                    return NativeCall.Impl.Int_Call_3(fp, descr, name, value);
+
+                if (dt == Runtime.PyWrapperDescriptorType
+                    || dt == Runtime.PyMethodType
+                    || typeof(ExtensionType).IsInstanceOfType(GetManagedObject(descr))
+                    )
+                {
+                    IntPtr fp = Marshal.ReadIntPtr(dt, TypeOffset.tp_descr_set);
+                    if (fp != IntPtr.Zero)
+                    {
+                        return NativeCall.Impl.Int_Call_3(fp, descr, name, value);
+                    }
+                    else
+                    {
+                        Exceptions.SetError(Exceptions.AttributeError,
+                            "attribute is read-only");
+                        return -1;
+                    }
                 }
-                Exceptions.SetError(Exceptions.AttributeError,
-                                    "attribute is read-only");
-                return -1;
-            }
-            
-            if (Runtime.PyObject_GenericSetAttr(tp, name, value) < 0) {
-                return -1;
             }
 
-            return 0;
+            var res = Runtime.PyObject_GenericSetAttr(tp, name, value);
+            Runtime.PyType_Modified(tp);
+
+            return res;
         }
 
         //====================================================================
@@ -220,12 +245,14 @@ namespace Python.Runtime {
         // here we just delegate to the generic type def implementation. Its
         // own mp_subscript
         //====================================================================
-        public static IntPtr mp_subscript(IntPtr tp, IntPtr idx) {
-             ClassBase cb = GetManagedObject(tp) as ClassBase;
-             if (cb != null) {
-                 return cb.type_subscript(idx);
-             }
-             return Exceptions.RaiseTypeError("unsubscriptable object");
+        public static IntPtr mp_subscript(IntPtr tp, IntPtr idx)
+        {
+            ClassBase cb = GetManagedObject(tp) as ClassBase;
+            if (cb != null)
+            {
+                return cb.type_subscript(idx);
+            }
+            return Exceptions.RaiseTypeError("unsubscriptable object");
         }
 
         //====================================================================
@@ -233,17 +260,19 @@ namespace Python.Runtime {
         // by this metatype is no longer referenced from the Python runtime.
         //====================================================================
 
-        public static void tp_dealloc(IntPtr tp) {
+        public static void tp_dealloc(IntPtr tp)
+        {
             // Fix this when we dont cheat on the handle for subclasses!
 
             int flags = (int)Marshal.ReadIntPtr(tp, TypeOffset.tp_flags);
-            if ((flags & TypeFlags.Subclass) == 0) {
+            if ((flags & TypeFlags.Subclass) == 0)
+            {
                 IntPtr gc = Marshal.ReadIntPtr(tp, TypeOffset.magic());
                 ((GCHandle)gc).Free();
             }
 
             IntPtr op = Marshal.ReadIntPtr(tp, TypeOffset.ob_type);
-            Runtime.Decref(op);
+            Runtime.XDecref(op);
 
             // Delegate the rest of finalization the Python metatype. Note
             // that the PyType_Type implementation of tp_dealloc will call
@@ -256,10 +285,53 @@ namespace Python.Runtime {
             return;
         }
 
+        static IntPtr DoInstanceCheck(IntPtr tp, IntPtr args, bool checkType)
+        {
+            ClassBase cb = GetManagedObject(tp) as ClassBase;
 
+            if (cb == null)
+            {
+                Runtime.XIncref(Runtime.PyFalse);
+                return Runtime.PyFalse;
+            }
 
+            using (PyList argsObj = new PyList(args))
+            {
+                if (argsObj.Length() != 1)
+                    return Exceptions.RaiseTypeError("Invalid parameter count");
 
+                PyObject arg = argsObj[0];
+                PyObject otherType;
+                if (checkType)
+                    otherType = arg;
+                else
+                    otherType = arg.GetPythonType();
+
+                if (Runtime.PyObject_TYPE(otherType.Handle) != PyCLRMetaType)
+                {
+                    Runtime.XIncref(Runtime.PyFalse);
+                    return Runtime.PyFalse;
+                }
+
+                ClassBase otherCb = GetManagedObject(otherType.Handle) as ClassBase;
+                if (otherCb == null)
+                {
+                    Runtime.XIncref(Runtime.PyFalse);
+                    return Runtime.PyFalse;
+                }
+
+                return Converter.ToPython(cb.type.IsAssignableFrom(otherCb.type));
+            }
+        }
+
+        public static IntPtr __instancecheck__(IntPtr tp, IntPtr args)
+        {
+            return DoInstanceCheck(tp, args, false);
+        }
+
+        public static IntPtr __subclasscheck__(IntPtr tp, IntPtr args)
+        {
+            return DoInstanceCheck(tp, args, true);
+        }
     }
-
-
 }
