@@ -1,35 +1,26 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
 using System.Runtime.InteropServices;
 
-namespace Python.Runtime {
-
+namespace Python.Runtime
+{
     /// <summary>
     /// Represents a Python float object. See the documentation at
     /// http://www.python.org/doc/current/api/floatObjects.html
     /// </summary>
-
-    public class PyFloat : PyNumber {
-
+    public class PyFloat : PyNumber
+    {
         /// <summary>
         /// PyFloat Constructor
         /// </summary>
         ///
         /// <remarks>
-        /// Creates a new PyFloat from an existing object reference. Note 
-        /// that the instance assumes ownership of the object reference. 
-        /// The object reference is not checked for type-correctness. 
+        /// Creates a new PyFloat from an existing object reference. Note
+        /// that the instance assumes ownership of the object reference.
+        /// The object reference is not checked for type-correctness.
         /// </remarks>
-
-        public PyFloat(IntPtr ptr) : base(ptr) {}
+        public PyFloat(IntPtr ptr) : base(ptr)
+        {
+        }
 
 
         /// <summary>
@@ -37,16 +28,17 @@ namespace Python.Runtime {
         /// </summary>
         ///
         /// <remarks>
-        /// Copy constructor - obtain a PyFloat from a generic PyObject. An 
+        /// Copy constructor - obtain a PyFloat from a generic PyObject. An
         /// ArgumentException will be thrown if the given object is not a
         /// Python float object.
         /// </remarks>
-
-        public PyFloat(PyObject o) : base() {
-            if (!IsFloatType(o)) {
+        public PyFloat(PyObject o) : base()
+        {
+            if (!IsFloatType(o))
+            {
                 throw new ArgumentException("object is not a float");
             }
-            Runtime.Incref(o.obj);
+            Runtime.XIncref(o.obj);
             obj = o.obj;
         }
 
@@ -58,10 +50,11 @@ namespace Python.Runtime {
         /// <remarks>
         /// Creates a new Python float from a double value.
         /// </remarks>
-
-        public PyFloat(double value) : base() {
+        public PyFloat(double value) : base()
+        {
             obj = Runtime.PyFloat_FromDouble(value);
-            if (obj == IntPtr.Zero) {
+            if (obj == IntPtr.Zero)
+            {
                 throw new PythonException();
             }
         }
@@ -74,12 +67,15 @@ namespace Python.Runtime {
         /// <remarks>
         /// Creates a new Python float from a string value.
         /// </remarks>
-
-        public PyFloat(string value) : base() {
-            PyString s = new PyString(value);
-            obj = Runtime.PyFloat_FromString(s.obj, IntPtr.Zero);
-            if (obj == IntPtr.Zero) {
-                throw new PythonException();
+        public PyFloat(string value) : base()
+        {
+            using (PyString s = new PyString(value))
+            {
+                obj = Runtime.PyFloat_FromString(s.obj, IntPtr.Zero);
+                if (obj == IntPtr.Zero)
+                {
+                    throw new PythonException();
+                }
             }
         }
 
@@ -91,8 +87,8 @@ namespace Python.Runtime {
         /// <remarks>
         /// Returns true if the given object is a Python float.
         /// </remarks>
-
-        public static bool IsFloatType(PyObject value) {
+        public static bool IsFloatType(PyObject value)
+        {
             return Runtime.PyFloat_Check(value.obj);
         }
 
@@ -103,20 +99,18 @@ namespace Python.Runtime {
         ///
         /// <remarks>
         /// <remarks>
-        /// Convert a Python object to a Python float if possible, raising  
+        /// Convert a Python object to a Python float if possible, raising
         /// a PythonException if the conversion is not possible. This is
         /// equivalent to the Python expression "float(object)".
         /// </remarks>
-
-        public static PyFloat AsFloat(PyObject value) {
+        public static PyFloat AsFloat(PyObject value)
+        {
             IntPtr op = Runtime.PyNumber_Float(value.obj);
-            if (op == IntPtr.Zero) {
+            if (op == IntPtr.Zero)
+            {
                 throw new PythonException();
             }
             return new PyFloat(op);
         }
-
-
     }
-
-}
+}
\ No newline at end of file
