@@ -1,12 +1,3 @@
-// ==========================================================================
-// This software is subject to the provisions of the Zope Public License,
-// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-// FOR A PARTICULAR PURPOSE.
-// ==========================================================================
-
 using System;
 using System.Runtime.InteropServices;
 using System.Reflection.Emit;
@@ -15,22 +6,21 @@ using System.Collections;
 using System.Reflection;
 using System.Threading;
 
-namespace Python.Runtime {
-
+namespace Python.Runtime
+{
     //=======================================================================
     // The TypeManager class is responsible for building binary-compatible
     // Python type objects that are implemented in managed code.
     //=======================================================================
 
-    internal class TypeManager {
-
+    internal class TypeManager
+    {
         static BindingFlags tbFlags;
         static Dictionary<Type, IntPtr> cache;
-        static int obSize;
 
-        static TypeManager() {
+        static TypeManager()
+        {
             tbFlags = BindingFlags.Public | BindingFlags.Static;
-            obSize = 5 * IntPtr.Size;
             cache = new Dictionary<Type, IntPtr>(128);
         }
 
@@ -42,12 +32,14 @@ namespace Python.Runtime {
         // descriptor and utility types like ModuleObject, PropertyObject, etc.
         //====================================================================
 
-        internal static IntPtr GetTypeHandle(Type type) {
+        internal static IntPtr GetTypeHandle(Type type)
+        {
             // Note that these types are cached with a refcount of 1, so they
             // effectively exist until the CPython runtime is finalized.
             IntPtr handle = IntPtr.Zero;
             cache.TryGetValue(type, out handle);
-            if (handle != IntPtr.Zero) {
+            if (handle != IntPtr.Zero)
+            {
                 return handle;
             }
             handle = CreateType(type);
@@ -62,10 +54,12 @@ namespace Python.Runtime {
         // the appropriate semantics in Python for the reflected managed type.
         //====================================================================
 
-        internal static IntPtr GetTypeHandle(ManagedType obj, Type type) {
+        internal static IntPtr GetTypeHandle(ManagedType obj, Type type)
+        {
             IntPtr handle = IntPtr.Zero;
             cache.TryGetValue(type, out handle);
-            if (handle != IntPtr.Zero) {
+            if (handle != IntPtr.Zero)
+            {
                 return handle;
             }
             handle = CreateType(obj, type);
@@ -83,14 +77,15 @@ namespace Python.Runtime {
         // do as much of the allocation and initialization work as possible.
         //====================================================================
 
-        internal static IntPtr CreateType(Type impl) {
-
+        internal static IntPtr CreateType(Type impl)
+        {
             IntPtr type = AllocateTypeObject(impl.Name);
+            int ob_size = ObjectOffset.Size(type);
 
             // Set tp_basicsize to the size of our managed instance objects.
-            Marshal.WriteIntPtr(type, TypeOffset.tp_basicsize, (IntPtr)obSize);
+            Marshal.WriteIntPtr(type, TypeOffset.tp_basicsize, (IntPtr)ob_size);
 
-            IntPtr offset = (IntPtr)ObjectOffset.ob_dict;
+            IntPtr offset = (IntPtr)ObjectOffset.DictOffset(type);
             Marshal.WriteIntPtr(type, TypeOffset.tp_dictoffset, offset);
 
             InitializeSlots(type, impl);
@@ -111,49 +106,59 @@ namespace Python.Runtime {
         }
 
 
-        internal static IntPtr CreateType(ManagedType impl, Type clrType) {
+        internal static IntPtr CreateType(ManagedType impl, Type clrType)
+        {
             // Cleanup the type name to get rid of funny nested type names.
             string name = "CLR." + clrType.FullName;
             int i = name.LastIndexOf('+');
-            if (i > -1) {
+            if (i > -1)
+            {
                 name = name.Substring(i + 1);
             }
             i = name.LastIndexOf('.');
-            if (i > -1) {
+            if (i > -1)
+            {
                 name = name.Substring(i + 1);
             }
 
             IntPtr base_ = IntPtr.Zero;
+            int ob_size = ObjectOffset.Size(Runtime.PyTypeType);
+            int tp_dictoffset = ObjectOffset.DictOffset(Runtime.PyTypeType);
+
             // XXX Hack, use a different base class for System.Exception
             // Python 2.5+ allows new style class exceptions but they *must*
             // subclass BaseException (or better Exception).
-#if (PYTHON25 || PYTHON26 || PYTHON27)
-            if (clrType == typeof(System.Exception)) {
+            if (typeof(System.Exception).IsAssignableFrom(clrType))
+            {
+                ob_size = ObjectOffset.Size(Exceptions.Exception);
+                tp_dictoffset = ObjectOffset.DictOffset(Exceptions.Exception);
+            }
+
+            if (clrType == typeof(System.Exception))
+            {
                 base_ = Exceptions.Exception;
-                Runtime.Incref(base_);
-            } else
-#endif
-            if (clrType.BaseType != null) {
+            }
+            else if (clrType.BaseType != null)
+            {
                 ClassBase bc = ClassManager.GetClass(clrType.BaseType);
                 base_ = bc.pyHandle;
             }
 
             IntPtr type = AllocateTypeObject(name);
 
-            Marshal.WriteIntPtr(type,TypeOffset.ob_type,Runtime.PyCLRMetaType);
-            Runtime.Incref(Runtime.PyCLRMetaType);
+            Marshal.WriteIntPtr(type, TypeOffset.ob_type, Runtime.PyCLRMetaType);
+            Runtime.XIncref(Runtime.PyCLRMetaType);
 
-            Marshal.WriteIntPtr(type, TypeOffset.tp_basicsize, (IntPtr)obSize);
+            Marshal.WriteIntPtr(type, TypeOffset.tp_basicsize, (IntPtr)ob_size);
             Marshal.WriteIntPtr(type, TypeOffset.tp_itemsize, IntPtr.Zero);
-
-            IntPtr offset = (IntPtr)ObjectOffset.ob_dict;
-            Marshal.WriteIntPtr(type, TypeOffset.tp_dictoffset, offset);
+            Marshal.WriteIntPtr(type, TypeOffset.tp_dictoffset, (IntPtr)tp_dictoffset);
 
             InitializeSlots(type, impl.GetType());
 
-            if (base_ != IntPtr.Zero) {
+            if (base_ != IntPtr.Zero)
+            {
                 Marshal.WriteIntPtr(type, TypeOffset.tp_base, base_);
-                Runtime.Incref(base_);
+                Runtime.XIncref(base_);
             }
 
             int flags = TypeFlags.Default;
@@ -188,70 +193,113 @@ namespace Python.Runtime {
             return type;
         }
 
-        internal static IntPtr CreateSubType(IntPtr args) {
-
-            IntPtr py_name = Runtime.PyTuple_GetItem(args, 0);
-            IntPtr bases = Runtime.PyTuple_GetItem(args, 1);
-            IntPtr dict = Runtime.PyTuple_GetItem(args, 2);
-            IntPtr base_ = Runtime.PyTuple_GetItem(bases, 0);
-
+        internal static IntPtr CreateSubType(IntPtr py_name, IntPtr py_base_type, IntPtr py_dict)
+        {
+            // Utility to create a subtype of a managed type with the ability for the
+            // a python subtype able to override the managed implementation
             string name = Runtime.GetManagedString(py_name);
-            IntPtr type = AllocateTypeObject(name);
 
-            Marshal.WriteIntPtr(type,TypeOffset.ob_type,Runtime.PyCLRMetaType);
-            Runtime.Incref(Runtime.PyCLRMetaType);
+            // the derived class can have class attributes __assembly__ and __module__ which
+            // control the name of the assembly and module the new type is created in.
+            object assembly = null;
+            object namespaceStr = null;
 
-            Marshal.WriteIntPtr(type, TypeOffset.tp_basicsize, (IntPtr)obSize);
-            Marshal.WriteIntPtr(type, TypeOffset.tp_itemsize, IntPtr.Zero);
+            List<PyObject> disposeList = new List<PyObject>();
+            try
+            {
+                PyObject assemblyKey = new PyObject(Converter.ToPython("__assembly__", typeof(String)));
+                disposeList.Add(assemblyKey);
+                if (0 != Runtime.PyMapping_HasKey(py_dict, assemblyKey.Handle))
+                {
+                    PyObject pyAssembly = new PyObject(Runtime.PyDict_GetItem(py_dict, assemblyKey.Handle));
+                    disposeList.Add(pyAssembly);
+                    if (!Converter.ToManagedValue(pyAssembly.Handle, typeof(String), out assembly, false))
+                        throw new InvalidCastException("Couldn't convert __assembly__ value to string");
+                }
 
-            IntPtr offset = (IntPtr)ObjectOffset.ob_dict;
-            Marshal.WriteIntPtr(type, TypeOffset.tp_dictoffset, offset);
+                PyObject namespaceKey = new PyObject(Converter.ToPythonImplicit("__namespace__"));
+                disposeList.Add(namespaceKey);
+                if (0 != Runtime.PyMapping_HasKey(py_dict, namespaceKey.Handle))
+                {
+                    PyObject pyNamespace = new PyObject(Runtime.PyDict_GetItem(py_dict, namespaceKey.Handle));
+                    disposeList.Add(pyNamespace);
+                    if (!Converter.ToManagedValue(pyNamespace.Handle, typeof(String), out namespaceStr, false))
+                        throw new InvalidCastException("Couldn't convert __namespace__ value to string");
+                }
+            }
+            finally
+            {
+                foreach (PyObject o in disposeList)
+                    o.Dispose();
+            }
 
-            IntPtr dc = Runtime.PyDict_Copy(dict);
-            Marshal.WriteIntPtr(type, TypeOffset.tp_dict, dc);
+            // create the new managed type subclassing the base managed type
+            ClassBase baseClass = ManagedType.GetManagedObject(py_base_type) as ClassBase;
+            if (null == baseClass)
+            {
+                return Exceptions.RaiseTypeError("invalid base class, expected CLR class type");
+            }
 
-            Marshal.WriteIntPtr(type, TypeOffset.tp_base, base_);
-            Runtime.Incref(base_);
+            try
+            {
+                Type subType = ClassDerivedObject.CreateDerivedType(name,
+                    baseClass.type,
+                    py_dict,
+                    (string)namespaceStr,
+                    (string)assembly);
 
-            int flags = TypeFlags.Default;
-            flags |= TypeFlags.Managed;
-            flags |= TypeFlags.HeapType;
-            flags |= TypeFlags.BaseType;
-            flags |= TypeFlags.Subclass;
-            flags |= TypeFlags.HaveGC;
-            Marshal.WriteIntPtr(type, TypeOffset.tp_flags, (IntPtr)flags);
+                // create the new ManagedType and python type
+                ClassBase subClass = ClassManager.GetClass(subType);
+                IntPtr py_type = GetTypeHandle(subClass, subType);
 
-            CopySlot(base_, type, TypeOffset.tp_traverse);
-            CopySlot(base_, type, TypeOffset.tp_clear);
-            CopySlot(base_, type, TypeOffset.tp_is_gc);
+                // by default the class dict will have all the C# methods in it, but as this is a
+                // derived class we want the python overrides in there instead if they exist.
+                IntPtr cls_dict = Marshal.ReadIntPtr(py_type, TypeOffset.tp_dict);
+                Runtime.PyDict_Update(cls_dict, py_dict);
 
-            Runtime.PyType_Ready(type);
-
-
-            IntPtr tp_dict = Marshal.ReadIntPtr(type, TypeOffset.tp_dict);
-            IntPtr mod = Runtime.PyString_FromString("CLR");
-            Runtime.PyDict_SetItemString(tp_dict, "__module__", mod);
-
-            // for now, move up hidden handle...
-            IntPtr gc = Marshal.ReadIntPtr(base_, TypeOffset.magic());
-            Marshal.WriteIntPtr(type, TypeOffset.magic(), gc);
-
-            return type;
+                return py_type;
+            }
+            catch (Exception e)
+            {
+                return Exceptions.RaiseTypeError(e.Message);
+            }
         }
 
+        internal static IntPtr WriteMethodDef(IntPtr mdef, IntPtr name, IntPtr func, int flags, IntPtr doc)
+        {
+            Marshal.WriteIntPtr(mdef, name);
+            Marshal.WriteIntPtr(mdef, (1*IntPtr.Size), func);
+            Marshal.WriteInt32(mdef, (2*IntPtr.Size), flags);
+            Marshal.WriteIntPtr(mdef, (3*IntPtr.Size), doc);
+            return mdef + 4*IntPtr.Size;
+        }
 
-        internal static IntPtr CreateMetaType(Type impl) {
+        internal static IntPtr WriteMethodDef(IntPtr mdef, string name, IntPtr func, int flags = 0x0001,
+            string doc = null)
+        {
+            IntPtr namePtr = Marshal.StringToHGlobalAnsi(name);
+            IntPtr docPtr = doc != null ? Marshal.StringToHGlobalAnsi(doc) : IntPtr.Zero;
 
+            return WriteMethodDef(mdef, namePtr, func, flags, docPtr);
+        }
+
+        internal static IntPtr WriteMethodDefSentinel(IntPtr mdef)
+        {
+            return WriteMethodDef(mdef, IntPtr.Zero, IntPtr.Zero, 0, IntPtr.Zero);
+        }
+
+        internal static IntPtr CreateMetaType(Type impl)
+        {
             // The managed metatype is functionally little different than the
             // standard Python metatype (PyType_Type). It overrides certain of
-            // the standard type slots, and has to subclass PyType_Type for 
+            // the standard type slots, and has to subclass PyType_Type for
             // certain functions in the C runtime to work correctly with it.
 
             IntPtr type = AllocateTypeObject("CLR Metatype");
             IntPtr py_type = Runtime.PyTypeType;
 
             Marshal.WriteIntPtr(type, TypeOffset.tp_base, py_type);
-            Runtime.Incref(py_type);
+            Runtime.XIncref(py_type);
 
             // Copy gc and other type slots from the base Python metatype.
 
@@ -275,6 +323,26 @@ namespace Python.Runtime {
             flags |= TypeFlags.HaveGC;
             Marshal.WriteIntPtr(type, TypeOffset.tp_flags, (IntPtr)flags);
 
+            // We need space for 3 PyMethodDef structs, each of them
+            // 4 int-ptrs in size.
+            IntPtr mdef = Runtime.PyMem_Malloc(3*(4*IntPtr.Size));
+            IntPtr mdefStart = mdef;
+            mdef = WriteMethodDef(
+                mdef,
+                "__instancecheck__",
+                Interop.GetThunk(typeof(MetaType).GetMethod("__instancecheck__"), "BinaryFunc")
+                );
+
+            mdef = WriteMethodDef(
+                mdef,
+                "__subclasscheck__",
+                Interop.GetThunk(typeof(MetaType).GetMethod("__subclasscheck__"), "BinaryFunc")
+                );
+
+            mdef = WriteMethodDefSentinel(mdef);
+
+            Marshal.WriteIntPtr(type, TypeOffset.tp_methods, mdefStart);
+
             Runtime.PyType_Ready(type);
 
             IntPtr dict = Marshal.ReadIntPtr(type, TypeOffset.tp_dict);
@@ -288,8 +356,8 @@ namespace Python.Runtime {
 
 
         internal static IntPtr BasicSubType(string name, IntPtr base_,
-                                            Type impl) {
-
+            Type impl)
+        {
             // Utility to create a subtype of a std Python type, but with
             // a managed type able to override implementation
 
@@ -304,7 +372,7 @@ namespace Python.Runtime {
             //Marshal.WriteIntPtr(type, TypeOffset.tp_dict, dc);
 
             Marshal.WriteIntPtr(type, TypeOffset.tp_base, base_);
-            Runtime.Incref(base_);
+            Runtime.XIncref(base_);
 
             int flags = TypeFlags.Default;
             flags |= TypeFlags.Managed;
@@ -332,18 +400,31 @@ namespace Python.Runtime {
         // Utility method to allocate a type object & do basic initialization.
         //====================================================================
 
-        internal static IntPtr AllocateTypeObject(string name) {
+        internal static IntPtr AllocateTypeObject(string name)
+        {
             IntPtr type = Runtime.PyType_GenericAlloc(Runtime.PyTypeType, 0);
 
             // Cheat a little: we'll set tp_name to the internal char * of
             // the Python version of the type name - otherwise we'd have to
             // allocate the tp_name and would have no way to free it.
-
+#if PYTHON3
+    // For python3 we leak two objects. One for the ascii representation
+    // required for tp_name, and another for the unicode representation
+    // for ht_name.
+            IntPtr temp = Runtime.PyBytes_FromString(name);
+            IntPtr raw = Runtime.PyBytes_AS_STRING(temp);
+            temp = Runtime.PyUnicode_FromString(name);
+#elif PYTHON2
             IntPtr temp = Runtime.PyString_FromString(name);
             IntPtr raw = Runtime.PyString_AS_STRING(temp);
+#endif
             Marshal.WriteIntPtr(type, TypeOffset.tp_name, raw);
             Marshal.WriteIntPtr(type, TypeOffset.name, temp);
 
+#if PYTHON3
+            Marshal.WriteIntPtr(type, TypeOffset.qualname, temp);
+#endif
+
             long ptr = type.ToInt64(); // 64-bit safe
 
             temp = new IntPtr(ptr + TypeOffset.nb_add);
@@ -355,8 +436,13 @@ namespace Python.Runtime {
             temp = new IntPtr(ptr + TypeOffset.mp_length);
             Marshal.WriteIntPtr(type, TypeOffset.tp_as_mapping, temp);
 
+#if PYTHON3
+            temp = new IntPtr(ptr + TypeOffset.bf_getbuffer);
+            Marshal.WriteIntPtr(type, TypeOffset.tp_as_buffer, temp);
+#elif PYTHON2
             temp = new IntPtr(ptr + TypeOffset.bf_getreadbuffer);
             Marshal.WriteIntPtr(type, TypeOffset.tp_as_buffer, temp);
+#endif
 
             return type;
         }
@@ -368,28 +454,33 @@ namespace Python.Runtime {
         // the Python object to the managed methods of the implementing Type.
         //====================================================================
 
-        internal static void InitializeSlots(IntPtr type, Type impl) {
+        internal static void InitializeSlots(IntPtr type, Type impl)
+        {
             Hashtable seen = new Hashtable(8);
             Type offsetType = typeof(TypeOffset);
 
-            while (impl != null) {
+            while (impl != null)
+            {
                 MethodInfo[] methods = impl.GetMethods(tbFlags);
-                for (int i = 0; i < methods.Length; i++) {
+                for (int i = 0; i < methods.Length; i++)
+                {
                     MethodInfo method = methods[i];
                     string name = method.Name;
-                    if (! (name.StartsWith("tp_") ||
-                           name.StartsWith("nb_") ||
-                           name.StartsWith("sq_") ||
-                           name.StartsWith("mp_") ||
-                           name.StartsWith("bf_")
-                           ) ) {
+                    if (!(name.StartsWith("tp_") ||
+                          name.StartsWith("nb_") ||
+                          name.StartsWith("sq_") ||
+                          name.StartsWith("mp_") ||
+                          name.StartsWith("bf_")
+                        ))
+                    {
                         continue;
                     }
 
-                    if (seen[name] != null) {
+                    if (seen[name] != null)
+                    {
                         continue;
                     }
-                    
+
                     FieldInfo fi = offsetType.GetField(name);
                     int offset = (int)fi.GetValue(offsetType);
 
@@ -401,7 +492,6 @@ namespace Python.Runtime {
 
                 impl = impl.BaseType;
             }
-
         }
 
 
@@ -411,24 +501,32 @@ namespace Python.Runtime {
         // to appear in the Python type __dict__ (based on custom attribute).
         //====================================================================
 
-        private static void InitMethods(IntPtr pytype, Type type) {
+        private static void InitMethods(IntPtr pytype, Type type)
+        {
             IntPtr dict = Marshal.ReadIntPtr(pytype, TypeOffset.tp_dict);
             Type marker = typeof(PythonMethodAttribute);
 
             BindingFlags flags = BindingFlags.Public | BindingFlags.Static;
+            HashSet<string> addedMethods = new HashSet<string>();
 
-            while (type != null) {
+            while (type != null)
+            {
                 MethodInfo[] methods = type.GetMethods(flags);
-                for (int i = 0; i < methods.Length; i++) {
+                for (int i = 0; i < methods.Length; i++)
+                {
                     MethodInfo method = methods[i];
-                    object[] attrs = method.GetCustomAttributes(marker, false);
-                    if (attrs.Length > 0) {
-                        string method_name = method.Name;
-                        MethodInfo[] mi = new MethodInfo[1];
-                        mi[0] = method;
-                        MethodObject m = new TypeMethod(method_name, mi);
-                        Runtime.PyDict_SetItemString(dict, method_name,
-                                                     m.pyHandle);
+                    if (!addedMethods.Contains(method.Name))
+                    {
+                        object[] attrs = method.GetCustomAttributes(marker, false);
+                        if (attrs.Length > 0)
+                        {
+                            string method_name = method.Name;
+                            MethodInfo[] mi = new MethodInfo[1];
+                            mi[0] = method;
+                            MethodObject m = new TypeMethod(type, method_name, mi);
+                            Runtime.PyDict_SetItemString(dict, method_name, m.pyHandle);
+                            addedMethods.Add(method_name);
+                        }
                     }
                 }
                 type = type.BaseType;
@@ -440,13 +538,10 @@ namespace Python.Runtime {
         // Utility method to copy slots from a given type to another type.
         //====================================================================
 
-        internal static void CopySlot(IntPtr from, IntPtr to, int offset) {
+        internal static void CopySlot(IntPtr from, IntPtr to, int offset)
+        {
             IntPtr fp = Marshal.ReadIntPtr(from, offset);
             Marshal.WriteIntPtr(to, offset, fp);
         }
-
-
     }
-
-
 }
