@@ -1,18 +1,18 @@
-# ===========================================================================
-# This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
-# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
-# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE.
-# ===========================================================================
 import clr
+
 clr.AddReference('Python.Test')
 clr.AddReference('System.Data')
 
 # testImplicitAssemblyLoad() passes on deprecation warning; perfect! #
 ##clr.AddReference('System.Windows.Forms')
 import sys, os, string, unittest, types, warnings
+from fnmatch import fnmatch
+import six
+
+if six.PY3:
+    ClassType = type
+else:
+    ClassType = types.ClassType
 
 
 class ModuleTests(unittest.TestCase):
@@ -22,10 +22,13 @@ class ModuleTests(unittest.TestCase):
         return type(object).__name__ == 'ModuleObject'
 
     def isCLRRootModule(self, object):
+        if six.PY3:
+            # in Python 3 the clr module is a normal python module
+            return object.__name__ == "clr"
         return type(object).__name__ == 'CLRModule'
 
     def isCLRClass(self, object):
-        return type(object).__name__ == 'CLR Metatype' # for now
+        return type(object).__name__ == 'CLR Metatype'  # for now
 
     def testAAAImportHookWorks(self):
         """Test that the import hook works correctly both using the
@@ -38,6 +41,10 @@ class ModuleTests(unittest.TestCase):
         import clr
         self.assertTrue(self.isCLRRootModule(clr))
 
+    def testVersionClr(self):
+        import clr
+        self.assertTrue(clr.__version__ >= "2.2.0")
+
     def testPreloadVar(self):
         import clr
         self.assertTrue(clr.getPreload() is False, clr.getPreload())
@@ -50,7 +57,7 @@ class ModuleTests(unittest.TestCase):
             self.assertTrue(clr.getPreload() is False, clr.getPreload())
             clr.setPreload(1)
             self.assertTrue(clr.getPreload() is True, clr.getPreload())
-            
+
             import System.Configuration
             content = dir(System.Configuration)
             self.assertTrue(len(content) > 10, content)
@@ -62,12 +69,15 @@ class ModuleTests(unittest.TestCase):
         import System
         self.assertEquals(type(System.__dict__), type({}))
         self.assertEquals(System.__name__, 'System')
-        self.assertEquals(System.__file__, None)
-        self.assertEquals(System.__doc__, None)
+        # the filename can be any module from the System namespace
+        # (eg System.Data.dll or System.dll, but also mscorlib.dll)
+        system_file = System.__file__
+        self.assertTrue(fnmatch(system_file, "*System*.dll") or fnmatch(system_file, "*mscorlib.dll"),
+                        "unexpected System.__file__: " + system_file)
+        self.assertTrue(System.__doc__.startswith("Namespace containing types from the following assemblies:"))
         self.assertTrue(self.isCLRClass(System.String))
         self.assertTrue(self.isCLRClass(System.Int32))
 
-
     def testSimpleImport(self):
         """Test simple import."""
         import System
@@ -78,10 +88,14 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(type(sys) == types.ModuleType)
         self.assertTrue(sys.__name__ == 'sys')
 
-        import httplib
-        self.assertTrue(type(httplib) == types.ModuleType)
-        self.assertTrue(httplib.__name__ == 'httplib')
-
+        if six.PY3:
+            import http.client as httplib
+            self.assertTrue(type(httplib) == types.ModuleType)
+            self.assertTrue(httplib.__name__ == 'http.client')
+        else:
+            import httplib
+            self.assertTrue(type(httplib) == types.ModuleType)
+            self.assertTrue(httplib.__name__ == 'httplib')
 
     def testSimpleImportWithAlias(self):
         """Test simple import with aliasing."""
@@ -93,10 +107,14 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(type(mySys) == types.ModuleType)
         self.assertTrue(mySys.__name__ == 'sys')
 
-        import httplib as myHttplib
-        self.assertTrue(type(myHttplib) == types.ModuleType)
-        self.assertTrue(myHttplib.__name__ == 'httplib')
-
+        if six.PY3:
+            import http.client as myHttplib
+            self.assertTrue(type(myHttplib) == types.ModuleType)
+            self.assertTrue(myHttplib.__name__ == 'http.client')
+        else:
+            import httplib as myHttplib
+            self.assertTrue(type(myHttplib) == types.ModuleType)
+            self.assertTrue(myHttplib.__name__ == 'httplib')
 
     def testDottedNameImport(self):
         """Test dotted-name import."""
@@ -108,7 +126,6 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(type(xml.dom) == types.ModuleType)
         self.assertTrue(xml.dom.__name__ == 'xml.dom')
 
-
     def testMultipleDottedNameImport(self):
         """Test an import bug with multiple dotted imports."""
         import System.Data
@@ -118,7 +135,6 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(self.isCLRModule(System.Data))
         self.assertTrue(System.Data.__name__ == 'System.Data')
 
-        
     def testDottedNameImportWithAlias(self):
         """Test dotted-name import with aliasing."""
         import System.Reflection as SysRef
@@ -129,7 +145,6 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(type(myDom) == types.ModuleType)
         self.assertTrue(myDom.__name__ == 'xml.dom')
 
-
     def testSimpleImportFrom(self):
         """Test simple 'import from'."""
         from System import Reflection
@@ -140,7 +155,6 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(type(dom) == types.ModuleType)
         self.assertTrue(dom.__name__ == 'xml.dom')
 
-
     def testSimpleImportFromWithAlias(self):
         """Test simple 'import from' with aliasing."""
         from System import Collections as Coll
@@ -151,14 +165,13 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(type(myDom) == types.ModuleType)
         self.assertTrue(myDom.__name__ == 'xml.dom')
 
-
     def testDottedNameImportFrom(self):
         """Test dotted-name 'import from'."""
         from System.Collections import Specialized
         self.assertTrue(self.isCLRModule(Specialized))
         self.assertTrue(
             Specialized.__name__ == 'System.Collections.Specialized'
-            )
+        )
 
         from System.Collections.Specialized import StringCollection
         self.assertTrue(self.isCLRClass(StringCollection))
@@ -169,10 +182,9 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(pulldom.__name__ == 'xml.dom.pulldom')
 
         from xml.dom.pulldom import PullDOM
-        self.assertTrue(type(PullDOM) == types.ClassType)
+        self.assertTrue(type(PullDOM) == ClassType)
         self.assertTrue(PullDOM.__name__ == 'PullDOM')
 
-
     def testDottedNameImportFromWithAlias(self):
         """Test dotted-name 'import from' with aliasing."""
         from System.Collections import Specialized as Spec
@@ -188,10 +200,9 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(myPulldom.__name__ == 'xml.dom.pulldom')
 
         from xml.dom.pulldom import PullDOM as myPullDOM
-        self.assertTrue(type(myPullDOM) == types.ClassType)
+        self.assertTrue(type(myPullDOM) == ClassType)
         self.assertTrue(myPullDOM.__name__ == 'PullDOM')
 
-
     def testFromModuleImportStar(self):
         """Test from module import * behavior."""
         count = len(locals().keys())
@@ -200,7 +211,6 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(self.isCLRModule(m))
         self.assertTrue(len(locals().keys()) > count + 1)
 
-
     def testImplicitAssemblyLoad(self):
         """Test implicit assembly loading via import."""
         with warnings.catch_warnings(record=True) as w:
@@ -223,7 +233,6 @@ class ModuleTests(unittest.TestCase):
             self.assertTrue(Form.__name__ == 'Form')
             self.assertEqual(len(w), 0)
 
-
     def testExplicitAssemblyLoad(self):
         """Test explicit assembly loading using standard CLR tools."""
         from System.Reflection import Assembly
@@ -231,14 +240,13 @@ class ModuleTests(unittest.TestCase):
 
         assembly = Assembly.LoadWithPartialName('System.Data')
         self.assertTrue(assembly != None)
-        
+
         import System.Data
-        self.assertTrue(sys.modules.has_key('System.Data'))
+        self.assertTrue('System.Data' in sys.modules)
 
         assembly = Assembly.LoadWithPartialName('SpamSpamSpamSpamEggsAndSpam')
         self.assertTrue(assembly == None)
 
-
     def testImplicitLoadAlreadyValidNamespace(self):
         """Test implicit assembly load over an already valid namespace."""
         # In this case, the mscorlib assembly (loaded by default) defines
@@ -250,24 +258,23 @@ class ModuleTests(unittest.TestCase):
         import System
         self.assertTrue(self.isCLRClass(System.UriBuilder))
 
-
     def testImportNonExistantModule(self):
         """Test import failure for a non-existant module."""
+
         def test():
             import System.SpamSpamSpam
 
         self.assertTrue(ImportError, test)
 
-
     def testLookupNoNamespaceType(self):
         """Test lookup of types without a qualified namespace."""
         import Python.Test
         import clr
         self.assertTrue(self.isCLRClass(clr.NoNamespaceType))
 
-
     def testModuleLookupRecursion(self):
         """Test for recursive lookup handling."""
+
         def test1():
             from System import System
 
@@ -279,14 +286,13 @@ class ModuleTests(unittest.TestCase):
 
         self.assertTrue(AttributeError, test2)
 
-
     def testModuleGetAttr(self):
         """Test module getattr behavior."""
         import System
 
         int_type = System.Int32
         self.assertTrue(self.isCLRClass(int_type))
-        
+
         module = System.Xml
         self.assertTrue(self.isCLRModule(module))
 
@@ -300,14 +306,13 @@ class ModuleTests(unittest.TestCase):
 
         self.assertTrue(TypeError, test)
 
-
     def testModuleAttrAbuse(self):
         """Test handling of attempts to set module attributes."""
 
         # It would be safer to use a dict-proxy as the __dict__ for CLR
         # modules, but as of Python 2.3 some parts of the CPython runtime
         # like dir() will fail if a module dict is not a real dictionary.
-        
+
         def test():
             import System
             System.__dict__['foo'] = 0
@@ -315,7 +320,6 @@ class ModuleTests(unittest.TestCase):
 
         self.assertTrue(test())
 
-
     def testModuleTypeAbuse(self):
         """Test handling of attempts to break the module type."""
         import System
@@ -337,32 +341,49 @@ class ModuleTests(unittest.TestCase):
         self.assertTrue(TypeError, test)
 
     def test_ClrListAssemblies(self):
-        from clr import ListAssemblies 
+        from clr import ListAssemblies
         verbose = list(ListAssemblies(True))
         short = list(ListAssemblies(False))
-        self.assertTrue(u'mscorlib' in short)
-        self.assertTrue(u'System' in short)
-        self.assertTrue('Culture=' in verbose[0])
-        self.assertTrue('Version=' in verbose[0])
+        self.assertTrue(six.u('mscorlib') in short)
+        self.assertTrue(six.u('System') in short)
+        self.assertTrue(six.u('Culture=') in verbose[0])
+        self.assertTrue(six.u('Version=') in verbose[0])
 
     def test_ClrAddReference(self):
         from clr import AddReference
         from System.IO import FileNotFoundException
         for name in ("System", "Python.Runtime"):
-            assy = AddReference(name) 
+            assy = AddReference(name)
             assyName = assy.GetName().Name
             self.assertEqual(assyName, name)
-        
-        self.assertRaises(FileNotFoundException, 
-            AddReference, "somethingtotallysilly")
+
+        self.assertRaises(FileNotFoundException,
+                          AddReference, "somethingtotallysilly")
+
+    def test_AssemblyLoadThreadSafety(self):
+        import time
+        from Python.Test import ModuleTest
+        # spin up .NET thread which loads assemblies and triggers AppDomain.AssemblyLoad event
+        ModuleTest.RunThreads()
+        time.sleep(1e-5)
+        for i in range(1, 100):
+            # call import clr, which in AssemblyManager.GetNames iterates through the loaded types
+            import clr
+            # import some .NET types
+            from System import DateTime
+            from System import Guid
+            from System.Collections.Generic import Dictionary
+            dict = Dictionary[Guid,DateTime]()
+        ModuleTest.JoinThreads()
 
 
 def test_suite():
     return unittest.makeSuite(ModuleTests)
 
+
 def main():
     unittest.TextTestRunner().run(test_suite())
 
+
 if __name__ == '__main__':
     main()
-
